"cen.sir" <-
function(y, delta, x, n.slice, joint.edrs, h, c=0.05){
    "cov1" <-
    function(x, use="pair"){
        n = nrow(x);
        c = cov(x, use=use)*(n-1)/n;
        c
    }
    
    "kernel.est" <-
    function(Bx,h){
        n = nrow(Bx);
        r = ncol(Bx);
        b = rep(h, r);
    
        kernel.x = matrix(rep(0,n*n), nrow=n);
        for (i in 1:n){
            Bx.temp = t(matrix(rep(Bx[i,],n), nrow=r, ncol=n));
            d2 = ((Bx - Bx.temp)/b)^2;
            ds = apply(d2, 1, sum);
            fh = exp(-0.5*(ds));
            kernel.x[i,] = fh;
        }
        kernel.x
    }
    
    "cond.sur.est" <-
    function(kernel.mtr, y, c){
        n = length(y);
        s.hat = rep(0, n)
        for (i in 1:n){
            yi = (1:n)[y>y[i]];
            fh = kernel.mtr[i,];
                if(length(yi)>0){
                    s.hat[i] = sum(fh[yi], na.rm=TRUE)/sum(fh, na.rm=TRUE);
                    s.hat[i] = max(s.hat[i], c, na.rm=TRUE);
                }else{
                    s.hat[i] = c;
                }
        }
        s.hat
    }

    "weight.est" <-
    function(t1, t2, index.x, y, delta, kernel.mtr, s.hat){
        if (t1 >= t2){
            cat(paste(t1, "\t", t2, "\n"));
                stop("t1 >= t2!");
        }
        
        # equation (5.2)
        fhatx0 = mean(kernel.mtr[index.x,], na.rm=TRUE)
        lambdahat.ttx.temp = kernel.mtr[index.x,] * (t1<y)*(y<t2) *  delta 
        lambdahat.ttx = mean((lambdahat.ttx.temp / s.hat), na.rm=TRUE) / fhatx0
    
        # equation (5.1)
        weight.ttx = exp(-lambdahat.ttx)
        weight.ttx
    }

    "slice.time" <-
    function(t, h){
        lt=length(t);
        n1=ceiling(lt/h);  
        n2=floor(lt/h);  
        if(n1==n2){
            n = lt/h;
            index = rep(n, h);
        }else if(n1 > n2){
            h1 = lt%%h;
            h2 = h - h1;
            index = rep(c(n1, n2), c(h1, h2));
        }
        t.slice=numeric(h+1);  # (h+1) time points
        t.slice[1]=min(t) - 0.000001;
        end = 0;
        for(i in 1:h){
            bgn = end + 1;
            end = end + index[i];
            t.slice[i+1] = t[end];
        }
        t.slice 
    }

    # order the data by survival time first
    ordr = order(y, -delta);
    y = y[ordr];
    x = x[ordr,];
    delta = delta[ordr];

    # first, estimate the kernel matrix and conditional survial function
    Bx = x %*% joint.edrs;
    kernel.mtr = kernel.est(Bx, h);
    s.hat = cond.sur.est(kernel.mtr, y, c);
    
    # sencondly, calculate the slice mean m_h
	t.slice = slice.time(y, n.slice);
	m = matrix(0, nrow=n.slice, ncol=ncol(x));
	p.hat = rep(1,(n.slice+1));
	p = rep(1,n.slice);
	e.hat.x = matrix(0, nrow=(n.slice+1), ncol=ncol(x));
	x.bar = apply(x, 2, mean, na.rm=TRUE);
	e.hat.x[1,] = x.bar;
	nobs = length(y);
	
	for(j in 1:(n.slice-1)){
		weight.vect = rep(1,nobs);
		idx = (1:nobs);
		idx.case1 = idx[y<t.slice[(j+1)] & delta==0 & s.hat>c];
		idx.case2 = idx[y>=t.slice[(j+1)]];
				
		for(i in idx.case1){
			weight.vect[i] = weight.est(y[i], t.slice[(j+1)], i, y, delta, 
                kernel.mtr, s.hat);
		}

		p.hat[(j+1)] = sum(weight.vect[idx.case1], na.rm=TRUE)/nobs;
		p.hat[(j+1)] = p.hat[(j+1)]+sum(weight.vect[idx.case2],na.rm=TRUE)/nobs;
	       	
        if(length(idx.case1)>1){
            x.case1 = x[idx.case1,]*weight.vect[idx.case1]
            ave.case1 = apply(x.case1, 2, sum, na.rm=TRUE)/nobs;
        }else if(length(idx.case1)==1){
            ave.case1 = x[idx.case1,]*weight.vect[idx.case1]/nobs;
        }else if(length(idx.case1)==0){
            ave.case1 = 0;
        }
        
        if(length(idx.case2)>1){
           ave.case2 = apply(x[idx.case2,], 2, sum, na.rm=TRUE)/nobs;
        }else if(length(idx.case2)==1){
           ave.case2 = x[idx.case2,]/nobs;
        }else if(length(idx.case2)==0){
           ave.case2 = 0;
        }
       
        e.hat.x[(j+1),] = ave.case1 + ave.case2;
        
        m[j,] = (e.hat.x[j,]-e.hat.x[(j+1),])/(p.hat[j]-p.hat[(j+1)]) - x.bar;
        p[j]  = p.hat[j] - p.hat[(j+1)];
	}
	
	m[n.slice,] = e.hat.x[n.slice,]/p.hat[n.slice] - x.bar;
	p[n.slice]  = p.hat[n.slice];

	# at last, compute the SIR direction
	mp = m*sqrt(p);
	sigma.eta = t(mp) %*% mp;
	sigma.x   = cov1(x, use="pair");
	eign = eigen.decomp(sigma.eta, sigma.x);
	
	ordr  = order(eign$values, decreasing=TRUE);
	e.val = eign$values[ordr];
	e.vec = eign$vectors[,ordr];
	
	if(is.null(colnames(x))){
	   x.names = paste("X", 1:ncol(x), sep="");
	}else{
	   x.names = colnames(x);
    }
    dimnames(e.vec) = list(x.names, paste("Dir", 1:ncol(e.vec), sep=""));
    
    csir = list(y=y, x=x, delta=delta, eval=e.val, evec=e.vec, nslice=n.slice);
    csir$c = c;
    csir$h = h;
    class(csir) = "csir";
    csir
}

